//
//
//   File: altmath.h       Created 12-MAY-2007      Richard B. Johnson
//                                                  RichardBJohnson@comcast.net
//
//   The data, text, and source-code within these files are presumed to
//   be accurate and free of encumbrances. Use at your own risk.
//

#ifndef _ALTMATH_H
#define _ALTMATH_H
#ifdef linux
#define ALIGN __asm__ __volatile__(".align 0x10\n");
#else
#define ALIGN _asm{align 10H}
#endif

enum WINDOW_TYPE {
     WINDOW_TYPE_RCOS,      // Raised cosine
     WINDOW_TYPE_HAM,       // Hamming
     WINDOW_TYPE_HAN,       // Hanning
     WINDOW_TYPE_NONE,      // Nothing
     WINDOW_TYPE_KAIS,      // Kaiser
     WINDOW_TYPE_ALOG,      // Johnson-Analogic
     WINDOW_TYPE_WEED,      // Weedon-Gauss
     WINDOW_TYPE_BKHA,      // Blackman-Harris
	 WINDOW_TYPE_BKMN,      // Blackman
	 WINDOW_TYPE_NUTA,      // Nutall
	 WINDOW_TYPE_BKNT,      // Blackman-Nuttall
	 WINDOW_TYPE_LANC       // Lanczos
     };

extern void parse_fft (
    double *a,              // Real part of data
    double *b,              // Imaginary part of data
    int m,                  // size of data = 2**m
    int iopt                // 0=dft, 1=idft
);

#if !defined(PI)
# define PI 3.1415926535897932384626433832795029  // c/d
#endif

#define ALPHA 9.675         // Alpha for Kaiser kernel
#define BADPI 3.10          // Used for noisey non-sync PI
#define HIGPI 3.20          // Used for noisey non-sync PI
#define PAD   0.4900        // For floating-point rounding
#define DYN   1e-12         // Min dynamic range number
#define ZERO  1e-50         // Save us from div/0

extern double ArrayRMS(double *buf, int len);
extern double IndexToFirstPeak(double *array, int len);
extern double MeasMean(double *array, int len);
extern double MeasStdDev(double *array, int len);
extern double MeasVariance(double *array, int len);
extern double do_FFT(double *out, double *input, enum WINDOW_TYPE typ, int len);
extern double get_bins(double *array, int ctr, int nr_bins);
extern double get_db(double ref, double val);
extern int NrCrossRefLevel(double *array, double ref, int len);
extern int SettlingTime(double *array, double ref, int len);
extern int TimeMaxSlope(double *array, int len);
extern int TimeToNegRef(double *array, double ref, int len);
extern int TimeToPosRef(double *array, double ref, int len);
extern int TimeToReference(double *array, double ref, int len);
extern int WithinLimits(double *array, double upper, double lower, int len);
extern int isNaN(double x);
extern void AbsValue(double *array, int len);
extern void AddConst(double *array, double val, int len);
extern void PutConst(double *array, double val, int len);
extern void CvtInt2Double(double *array, int *input, int len);
extern void CvtShort2Double(double *array, short *input, int len);
extern void DivConst(double *array, double val, int len);
extern void filter(double *buf, int box, int len);
extern void Log10(double *array, int len);
extern void LogN(double *array, int len);
extern void MakeAnalogicWIN(double *array, int len);
extern void MakeBlackmanHarris(double *array, int len);
extern void MakeBlackman(double *array, int len);
extern void MakeHammingWIN(double *array, int len);
extern void MakeHanningWIN(double *array, int len);
extern void MakeKaiserWIN(double *array, int len);
extern void MakeRaisedCOS(double *array, int len);
extern void MakeWeedonGauss(double *array, int len);
extern void MakeNutall(double *array, int len);
extern void MakeBlackNutall(double *array, int len);
extern void MakeLanczos(double *array, int len);
extern void MakeRectWIN(double *array, int len);
extern void MulConst(double *array, double val, int len);
extern void MultArray(double *array, double *kern, int len);
extern void SubConst(double *array, double val, int len);
extern void Window(double *array, enum WINDOW_TYPE typ, int len);
extern void get_phase(double *out, double *a, double *b, int len);
extern void get_mag(double *out, double *a, double *b, int len);

#endif



