//
//
//   File: fixtext.c     Created 12-MAY-2007        Richard B. Johnson
//                                                  RichardBJohnson@comcast.net
//
//   The data, text, and source-code within these files are presumed to
//   be accurate and free of encumbrances. Use at your own risk.
//

#ifdef _MSC_VER
#define WINDOWS
#else
#define LINUX
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#ifdef WINDOWS
extern int access(const char *, int);
#include <io.h>
#endif
#ifdef LINUX
#include <unistd.h>
#include <sys/time.h>
#define O_BINARY 0
#endif
#define FAIL -1
#define READ_LEN 0x400
#define WRITE_LEN (READ_LEN * 8)  /* Tab expansion could do this */
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-*/
/*
 *  Function prototypes.
 */
int main(int argc, char *argv[]);
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-*/
int main(int argc, char *argv[])
{
    char *inbuf, *outbuf;
    char outnam[0x400];
    int infd, outfd, stat, i, col;
    size_t len;
    char *ip, *op, c;
    int crlf = access("/usr/include/stdio.h", 0);
    if(argc < 2)
    {
        fprintf(stderr, "Usage:\n%s <infile> [outfile]\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    if((infd = open(argv[1], O_RDONLY|O_BINARY)) == FAIL)
    {
        fprintf(stderr, "Can't open file %s\n", argv[1]);
        exit(EXIT_FAILURE);
    }
    if(argc > 2)
        strncpy(outnam,  argv[2], sizeof(outnam));
    else
    {
        strncpy(outnam, argv[1], sizeof(outnam));
        strncat(outnam, ".tmp" , sizeof(outnam));
    }
    if((outfd = open(outnam, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0644)) == FAIL)
    {
        fprintf(stderr, "Can't create file %s\n", outnam);
        (void)close(infd);
        exit(EXIT_FAILURE);
    }
    inbuf  = (char *) malloc(READ_LEN  * sizeof(char));
    outbuf = (char *) malloc(WRITE_LEN * sizeof(char));
    if(!(inbuf && outbuf))
    {
        fprintf(stderr, "Memory allocation failure\n");
        exit(EXIT_FAILURE);
    }
    col = 0;
    while((stat = read(infd, inbuf, READ_LEN)) > 0)
    {
        ip = inbuf;
        op = outbuf;
        for(i=0; i< stat; i++)
        {
            if((c = *ip++) == '\r')         /* Filter all <CR> characters */
                continue;
            if(c == '\t')      /* A TAB character */
            {
                do {
                     *op++ = ' ';
                   } while (++col & 0x07);
                continue;
            }
            if(c == '\n')
            {
                while((op > outbuf) && (op[-1] == ' '))
                    op--;
                if(crlf)           /* Windoze stuff     */
                    *op++ = '\r';  /* Need a <CR> first */
                col = -1;
            }
            col++;
            *op++ = c;
        }
        len = (op - outbuf);
        op = outbuf;
        while(len)
        {
            if((stat = write(outfd, op, len)) == FAIL)
            {
                fprintf(stderr, "Can't write file %s\n", argv[2]);
                exit(EXIT_FAILURE);
            }
            op  += stat;
            len -= stat;
        }
    }
    if(close(infd) == FAIL)
    {
        fprintf(stderr, "Can't close file %s\n", argv[1]);
        exit(EXIT_FAILURE);
    }
    if(close(outfd) == FAIL)
    {
        fprintf(stderr, "Can't close file %s\n", outnam);
        exit(EXIT_FAILURE);
    }
    if(argc > 2)
        return 0;
    if(remove(argv[1]) == FAIL)
    {
        fprintf(stderr, "File %s can't be deleted.\nFile %s remains.\n",
                         argv[1], outnam);
        exit(EXIT_FAILURE);
    }
    if(rename(outnam, argv[1]) == FAIL)
    {
        fprintf(stderr, "Can't rename file %s to %s\n", argv[1], outnam);
        exit(EXIT_FAILURE);
    }
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/







