//
//
//   File: thd.c        Created 12-MAY-2007         Richard B. Johnson
//                                                  RichardBJohnson@comcast.net
//
//   The data, text, and source-code within these files are presumed to
//   be accurate and free of encumbrances. Use at your own risk.
//
//
//   Test THD.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "altmath.h"
#include "MeasureLibrary.h"


#define LEN 0x10000 
#define MAG 1000.0
#define DIS 0.01
#define NODES 0x80

#if !defined (M_PI)
#define M_PI PI
#endif

int main(void);


int main()
{
    double *d, f, theta, sample_rate, twopi;
    size_t i;

    sample_rate = (double) (LEN/NODES);
    twopi = M_PI * 2.0 * sqrt(2.7182818); /* Make asynchronous */
    if((d = (double *) malloc(LEN * sizeof(double))) == NULL)
    {
        fprintf(stderr, "Memory allocation error!\n");
        exit(EXIT_FAILURE);
    }

    f = 0.0;
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi;
        d[i] = sin(theta) * MAG;
    }
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 2.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 3.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 4.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 5.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 6.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 7.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    for(i=0; i< LEN; i++)
    {
        theta = ((double)i / sample_rate) * twopi * 8.0;
        d[i] += (sin(theta) * MAG * DIS);
    }
    f += (MAG * DIS);
    printf("The THD should be   : %f\n", f * 100.0/MAG);
    f = MeasureTHD(d, (double)NODES, sample_rate, LEN);
    printf("The THD measures    : %f\n", f);
//
//  Now add some noise.
//
        for(i=0; i< LEN; i++)
            d[i] += ((((double)rand()/(double)RAND_MAX) - 0.50) * MAG * DIS);
    f = MeasureTHD(d, (double)NODES, sample_rate, LEN);
    printf("The THD with noise  : %f\n", f);
//
//   Now add some noise spikes!
//
        for(i=1; i< LEN; i++)
            if(!(rand() % i))
               d[i] =  2.0 * MAG;
    f = MeasureTHD(d, (double)NODES, sample_rate, LEN);
    printf("The THD with spikes : %f\n", f);
//
//  This adds 100% offset.
//
        for(i=0; i< LEN; i++)
            d[i] -= MAG;
    f = MeasureTHD(d, (double)NODES, sample_rate, LEN);
    printf("The THD with offset : %f\n", f);
    free(d);
    return 0;
}


